@@ -28,7 +28,7 @@ Example (Standard Lightning Pattern):
     >>> import pandas as pd
     >>> 
     >>> # Step 1: Load and preprocess data
-    >>> df = pd.read_csv('data/sample_data.csv')
+    >>> df = pd.read_csv('data/your_data.csv')
     >>> df_processed = df[[col for col in df.columns if col != 'date']]
     >>> 
     >>> # Step 2: Create DataModule
@@ -40,7 +40,7 @@ Example (Standard Lightning Pattern):
     >>> model.load_config('config/default.yaml')
     >>> 
     >>> # Step 4: Create trainer and fit (standard Lightning pattern)
-    >>> trainer = DFMTrainer(max_epochs=100)
+    >>> trainer = DFMTrainer(max_epochs=100)  # DEFAULT_MAX_EPOCHS
     >>> trainer.fit(model, dm)
     >>> 
     >>> # Step 5: Predict
@@ -55,17 +55,17 @@ Example (Standard Lightning Pattern):
     >>> ddfm_model = DDFM(encoder_layers=[64, 32], num_factors=2)
     >>> ddfm_model.load_config('config/default.yaml')
     >>> 
-    >>> trainer_ddfm = DDFMTrainer(max_epochs=100)
+    >>> trainer_ddfm = DDFMTrainer(max_epochs=100)  # DEFAULT_MAX_EPOCHS
     >>> trainer_ddfm.fit(ddfm_model, dm_ddfm)
     >>> Xf, Zf = ddfm_model.predict(horizon=6)
     
-Note: DFMConfig and SeriesConfig are internal implementation details.
+Note: DFMConfig uses frequency dict to specify series (column names -> frequencies).
     Users should use Hydra YAML configuration files instead.
 
 For detailed documentation, see the README.md file and the tutorial notebooks/scripts.
 """
 
-__version__ = "0.4.9"
+__version__ = "0.5.55"
 
 # ============================================================================
 # PUBLIC API DEFINITION
@@ -75,7 +75,7 @@ __version__ = "0.4.9"
 # Internal reorganization should not break these imports.
 #
 # Public API categories:
-# 1. Configuration: DFMConfig, SeriesConfig, config sources
+# 1. Configuration: DFMConfig, config sources
 # 2. High-level API: DFM, DDFM, module-level convenience functions
 # 3. Core utilities: TimeIndex, diagnostics
 # 4. Models: BaseFactorModel, DDFM (low-level)
@@ -84,72 +84,77 @@ __version__ = "0.4.9"
 
 # Configuration (from config/ subpackage)
 from .config import (
-    DEFAULT_BLOCK_NAME,
-    ConfigSource, YamlSource, DictSource, HydraSource,
-    MergedConfigSource, make_config_source,
+    # DEFAULT_BLOCK_NAME,  # Removed to avoid circular import - import directly from functional.dfm_block
+    ConfigSource, YamlSource,
+    make_config_source,
 )
-# Internal imports (for backward compatibility, but not recommended)
-from .config import DFMConfig, SeriesConfig  # Internal use only
+# Internal imports (internal use only)
+from .config import DFMConfig
 
 # Results
-from .config.results import DFMResult, DDFMResult, BaseResult
+from .config import DFMResult, DDFMResult, KDFMResult, BaseResult
 
 # Utilities (from utils/ subpackage)
-from .utils.diagnostics import diagnose_series, print_series_diagnosis
-from .utils.time import calculate_rmse
+from .utils.metric import calculate_rmse
 
-# PyTorch Lightning modules (mandatory dependency)
+# DataModule classes (includes both Lightning and custom implementations)
 # Users can import these directly from dfm_python
-from .lightning import (
+from .datamodule import (
     DFMDataModule,
-    DFMDataset,  # Dataset class (usually not needed directly)
-    KalmanFilter,  # Module class
-    EMAlgorithm,  # Module class
+    DDFMDataModule,  # DDFM-specific DataModule
+    KDFMDataModule,  # KDFM-specific DataModule
+    DDFMDataset,  # DDFM Dataset class (usually not needed directly)
+    KDFMDataset,  # KDFM Dataset class (usually not needed directly)
 )
 
-# Nowcasting removed - use src.nowcast instead
-
 # Model implementations
 from .models.base import BaseFactorModel
 from .models.dfm import DFM
 
-# DDFM high-level API and low-level model (PyTorch is mandatory)
-from .models.ddfm import DDFM, DDFMModel
+# DDFM high-level API (PyTorch is mandatory)
+from .models.ddfm import DDFM
+
+# KDFM high-level API
+from .models.kdfm import KDFM
 
 __all__ = [
     # Core classes
     'DFM',
     # Model base and implementations
     'BaseFactorModel',
-    # Constants
-    'DEFAULT_BLOCK_NAME',
     # Config sources
-    'ConfigSource', 'YamlSource', 'DictSource', 'HydraSource',
-    'MergedConfigSource', 'make_config_source',
+    'ConfigSource', 'YamlSource',
+    'make_config_source',
     # Low-level API (functional interface - advanced usage)
-    'BaseResult', 'DFMResult', 'DDFMResult', 'calculate_rmse', 'diagnose_series', 'print_series_diagnosis',
+    'BaseResult', 'DFMResult', 'DDFMResult', 'KDFMResult', 'calculate_rmse',
 ]
 
 # DDFM high-level API (PyTorch is mandatory)
 __all__.extend([
     'DDFM',  # High-level API class
-    'DDFMModel',  # Low-level implementation (used internally)
+])
+
+# KDFM high-level API
+__all__.extend([
+    'KDFM',  # High-level API class
 ])
 
 # Lightning modules (mandatory dependency)
 __all__.extend([
     'DFMDataModule',
-    'DFMDataset',  # Dataset class (usually not needed directly)
-    'KalmanFilter',  # Module class
-    'EMAlgorithm',  # Module class
+    'DDFMDataModule',  # DDFM-specific DataModule
+    'KDFMDataModule',  # KDFM-specific DataModule
+    'DDFMDataset',  # DDFM Dataset class (usually not needed directly)
+    'KDFMDataset',  # KDFM Dataset class (usually not needed directly)
 ])
 
 # Trainer classes (mandatory dependency)
-from .trainer import DFMTrainer, DDFMTrainer
+from .trainer import DFMTrainer, DDFMTrainer, KDFMTrainer
 
 __all__.extend([
     'DFMTrainer',
     'DDFMTrainer',
+    'KDFMTrainer',
 ])
 
 
