@@ -68,6 +68,10 @@ from ..constants import (
     DEFAULT_NAN_METHOD,
     DEFAULT_NAN_K,
     DEFAULT_CLOCK_FREQUENCY,
+    DEFAULT_KDFM_AR_ORDER,
+    DEFAULT_KDFM_MA_ORDER,
+    FREQUENCY_HIERARCHY,
+    DEFAULT_HIERARCHY_VALUE,
 )
 
 
@@ -124,7 +128,7 @@ class BaseModelConfig:
     # Shared Data Handling Parameters
     # ========================================================================
     clock: str = 'm'  # Required: Base frequency for latent factors (global clock): 'd', 'w', 'm', 'q', 'sa', 'a' (defaults to 'm' for monthly)
-    target_scaler: Optional[ScalerType] = None  # Sklearn scaler instance (StandardScaler, RobustScaler, etc.) for target series only. Pass scaler object directly, not string. Feature series are assumed to be manually preprocessed.
+    target_scaler: Optional[ScalerType] = None  # Fitted sklearn scaler instance (StandardScaler, RobustScaler, etc.) for target series only. Must be a fitted scaler object (call .fit() on target data first). Pass scaler object directly, not string. Feature series are assumed to be manually preprocessed. If None, target series are assumed to be already in the desired scale.
     # Note: nan_method and nan_k are internal constants (DEFAULT_NAN_METHOD, DEFAULT_NAN_K) used during initialization only
     
     def __post_init__(self):
@@ -254,47 +258,38 @@ class DFMConfig(BaseModelConfig):
     # ========================================================================
     # EM Algorithm Parameters (HOW - controls the algorithm)
     # ========================================================================
-    ar_lag: int = 1  # Number of lags in AR transition equation (lookback window). Must be 1 or 2 (maximum supported order is VAR(2))
+    # Note: ar_lag removed - factors always use AR(1) dynamics (simplified)
     threshold: float = DEFAULT_EM_THRESHOLD  # EM convergence threshold
     max_iter: int = DEFAULT_EM_MAX_ITER  # Maximum EM iterations
     
     # ========================================================================
     # Numerical Stability Parameters (transparent and configurable)
     # ========================================================================
-    # AR Coefficient Clipping
-    clip_ar_coefficients: bool = True  # Enable AR coefficient clipping for stationarity
-    ar_clip_min: float = AR_CLIP_MIN  # Minimum AR coefficient (must be > -1 for stationarity)
-    ar_clip_max: float = AR_CLIP_MAX   # Maximum AR coefficient (must be < 1 for stationarity)
-    warn_on_ar_clip: bool = True  # Warn when AR coefficients are clipped (indicates near-unit root)
+    # AR Coefficient Clipping: If provided (not None), automatically enables clipping and always warns
+    ar_clip: Optional[Dict[str, float]] = None  # {"min": float, "max": float} - AR coefficient clipping bounds. If None, no clipping. If provided, clipping enabled and warnings always shown.
     
-    # Data Value Clipping
-    clip_data_values: bool = True  # Enable clipping of extreme data values
-    data_clip_threshold: float = 100.0  # Clip values beyond this many standard deviations
-    warn_on_data_clip: bool = True  # Warn when data values are clipped (indicates outliers)
+    # Data Value Clipping: If provided (not None), automatically enables clipping and always warns
+    data_clip: Optional[float] = None  # Clip values beyond this many standard deviations. If None, no clipping. If provided, clipping enabled and warnings always shown.
     
-    # Regularization
-    use_regularization: bool = True  # Enable regularization for numerical stability
-    regularization_scale: float = DEFAULT_REGULARIZATION_SCALE  # Scale factor for ridge regularization (relative to trace, default 1e-5)
-    min_eigenvalue: float = 1e-8  # Minimum eigenvalue for positive definite matrices
-    max_eigenvalue: float = 1e6   # Maximum eigenvalue cap to prevent explosion
-    warn_on_regularization: bool = True  # Warn when regularization is applied
+    # Regularization: If provided (not None), automatically enables regularization and always warns
+    regularization: Optional[Dict[str, float]] = None  # {"scale": float, "min_eigenvalue": float, "max_eigenvalue": float} - Regularization parameters. If None, no regularization. If provided, regularization enabled and warnings always shown.
     
-    # Damped Updates
-    use_damped_updates: bool = True  # Enable damped updates when likelihood decreases
-    damping_factor: float = 0.8  # Damping factor (0.8 = 80% new, 20% old)
-    warn_on_damped_update: bool = True  # Warn when damped updates are used
+    # Damped Updates: If provided (not None), automatically enables damping and always warns
+    damping_factor: Optional[float] = None  # Damping factor (0.8 = 80% new, 20% old). If None, no damping. If provided, damping enabled and warnings always shown.
     
-    # Idiosyncratic Component Augmentation
-    augment_idio: bool = True  # Enable state augmentation with idiosyncratic components (default: True)
-    augment_idio_slow: bool = True  # Enable tent-length chains for slower-frequency series (default: True)
+    # Idiosyncratic Component Parameters (auto-detected from frequencies)
     idio_rho0: float = DEFAULT_IDIO_RHO0  # Initial AR coefficient for idiosyncratic components (default: 0.1)
-    idio_min_var: float = 1e-8  # Minimum variance for idiosyncratic innovation covariance (default: 1e-8)
+    idio_min_var: float = MIN_DIAGONAL_VARIANCE  # Minimum variance for idiosyncratic innovation covariance (defaults to MIN_DIAGONAL_VARIANCE)
+    # Note: augment_idio and augment_idio_slow are auto-detected from frequency configuration
+    # - If all series use clock frequency: augment_idio=False, augment_idio_slow=False
+    # - If mixed frequencies detected: augment_idio=True, augment_idio_slow=True (auto-enabled)
     
     def __post_init__(self):
         """Validate blocks structure and derive block properties."""
         super().__post_init__()
         
         from ...config.adapter import _raise_config_error
+        from ..constants import FREQUENCY_HIERARCHY, DEFAULT_HIERARCHY_VALUE
         
         if not self.blocks:
             _raise_config_error("DFM configuration must contain at least one block.")
@@ -327,6 +322,67 @@ class DFMConfig(BaseModelConfig):
         from ...config.adapter import _raise_config_error
         if any(f < 1 for f in self.factors_per_block):
             _raise_config_error("factors_per_block must contain positive integers >= 1")
+        
+        # Auto-detect mixed frequencies and set augment_idio/augment_idio_slow
+        # If frequency dict exists, check if all frequencies match clock
+        if self.frequency is not None and len(self.frequency) > 0:
+            frequencies = list(self.frequency.values())
+            clock_hierarchy = FREQUENCY_HIERARCHY.get(self.clock, DEFAULT_HIERARCHY_VALUE)
+            is_mixed_freq = any(
+                FREQUENCY_HIERARCHY.get(freq, DEFAULT_HIERARCHY_VALUE) != clock_hierarchy
+                for freq in frequencies
+            )
+            # Auto-set augment_idio and augment_idio_slow based on frequency detection
+            # Store as internal attributes (not in __init__ signature)
+            object.__setattr__(self, '_augment_idio', is_mixed_freq)
+            object.__setattr__(self, '_augment_idio_slow', is_mixed_freq)
+        else:
+            # No frequency info yet - will be auto-detected when data is loaded
+            object.__setattr__(self, '_augment_idio', False)
+            object.__setattr__(self, '_augment_idio_slow', False)
+    
+    @property
+    def augment_idio(self) -> bool:
+        """Auto-detected: True if mixed frequencies detected, False if single frequency."""
+        return getattr(self, '_augment_idio', False)
+    
+    @property
+    def augment_idio_slow(self) -> bool:
+        """Auto-detected: True if mixed frequencies detected, False if single frequency."""
+        return getattr(self, '_augment_idio_slow', False)
+    
+    def _update_idio_flags_from_frequencies(self, frequencies: List[str]) -> None:
+        """Update augment_idio flags based on detected frequencies (called when data is loaded)."""
+        from ..constants import FREQUENCY_HIERARCHY, DEFAULT_HIERARCHY_VALUE
+        clock_hierarchy = FREQUENCY_HIERARCHY.get(self.clock, DEFAULT_HIERARCHY_VALUE)
+        is_mixed_freq = any(
+            FREQUENCY_HIERARCHY.get(freq, DEFAULT_HIERARCHY_VALUE) != clock_hierarchy
+            for freq in frequencies
+        )
+        object.__setattr__(self, '_augment_idio', is_mixed_freq)
+        object.__setattr__(self, '_augment_idio_slow', is_mixed_freq)
+    
+    def to_em_config(self) -> 'EMConfig':
+        """Create EMConfig from DFMConfig consolidated parameters."""
+        from ...functional.em import EMConfig
+        from ..constants import DEFAULT_REGULARIZATION, VAR_STABILITY_THRESHOLD
+        
+        # Extract regularization parameters directly from dict
+        if self.regularization is not None and isinstance(self.regularization, dict):
+            reg_scale = self.regularization.get('scale', DEFAULT_REGULARIZATION_SCALE)
+            min_eigenval = self.regularization.get('min_eigenvalue', MIN_EIGENVALUE)
+            max_eigenval = self.regularization.get('max_eigenvalue', VAR_STABILITY_THRESHOLD)
+        else:
+            reg_scale = DEFAULT_REGULARIZATION
+            min_eigenval = MIN_EIGENVALUE
+            max_eigenval = VAR_STABILITY_THRESHOLD
+        
+        return EMConfig(
+            regularization=reg_scale,
+            min_norm=min_eigenval,
+            max_eigenval=max_eigenval,
+            # Other parameters use defaults from EMConfig
+        )
     
     def get_blocks_array(self, columns: Optional[List[str]] = None) -> np.ndarray:
         """Get blocks as numpy array (N x B) where N is number of series and B is number of blocks.
@@ -357,40 +413,45 @@ class DFMConfig(BaseModelConfig):
         return self._cached_blocks
     
     @classmethod
-    def _extract_dfm_params(cls, data: Dict[str, Any]) -> Dict[str, Any]:
-        """Extract DFM-specific parameters from config dict."""
-        base_params = cls._extract_base(data)
-        dfm_params = cls._extract_params(data, {
-            'ar_lag': 1,
-            'threshold': DEFAULT_EM_THRESHOLD,
-            'max_iter': DEFAULT_EM_MAX_ITER,
-            'clip_ar_coefficients': True,
-            'ar_clip_min': -0.99,
-            'ar_clip_max': 0.99,
-            'use_regularization': True,
-            'regularization_scale': 1e-5,
-            'min_eigenvalue': MIN_EIGENVALUE,
-            'max_eigenvalue': MAX_EIGENVALUE,
-            'augment_idio': True,
-            'augment_idio_slow': True,
-            'idio_rho0': 0.1,
-            'idio_min_var': MIN_DIAGONAL_VARIANCE,
-        })
-        base_params.update(dfm_params)
-        return base_params
+    def _extract_dfm_params(cls, data: Dict[str, Any], base_params: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
+        """Extract DFM-specific parameters from config dict.
+        
+        Parameters
+        ----------
+        data : Dict[str, Any]
+            Config dictionary
+        base_params : Dict[str, Any], optional
+            Pre-extracted base parameters. If provided, avoids duplicate extraction.
+            If None, extracts base params internally.
+        """
+        if base_params is None:
+            base_params = cls._extract_base(data)
+        
+        # Extract consolidated parameters (new format only)
+        dfm_params = {
+            'threshold': data.get('threshold', DEFAULT_EM_THRESHOLD),
+            'max_iter': data.get('max_iter', DEFAULT_EM_MAX_ITER),
+            'idio_rho0': data.get('idio_rho0', DEFAULT_IDIO_RHO0),
+            'idio_min_var': data.get('idio_min_var', MIN_DIAGONAL_VARIANCE),
+            'ar_clip': data.get('ar_clip', None),
+            'data_clip': data.get('data_clip', None),
+            'regularization': data.get('regularization', None),
+            'damping_factor': data.get('damping_factor', None),
+        }
+        
+        result = base_params.copy()
+        result.update(dfm_params)
+        return result
     
     @classmethod
-    def from_dict(cls, data: Dict[str, Any]) -> Union['DFMConfig', 'DDFMConfig']:
-        """Create DFMConfig or DDFMConfig from dictionary.
+    def from_dict(cls, data: Dict[str, Any]) -> Union['DFMConfig', 'DDFMConfig', 'KDFMConfig']:
+        """Create DFMConfig, DDFMConfig, or KDFMConfig from dictionary.
         
-        Handles multiple formats:
-        1. New format: {'frequency': {'column_name': 'frequency'}, ...}
-        2. Legacy format (Hydra): {'series': {'series_id': {...}}, 'blocks': {...}}
-        3. Legacy format (list): {'series': [{'series_id': ..., 'frequency': ...}], ...}
+        Expected format: {'frequency': {'column_name': 'frequency'}, 'blocks': {...}, ...}
         
-        Also accepts estimation parameters: ar_lag, threshold, max_iter, etc.
+        Also accepts estimation parameters: threshold, max_iter, etc.
         """
-        from ...config.adapter import detect_config_type, MODEL_TYPE_DDFM, _normalize_blocks_dict
+        from ...config.adapter import detect_config_type, MODEL_TYPE_DDFM, MODEL_TYPE_KDFM, _normalize_blocks_dict
         from ...utils.errors import ConfigurationError
         
         # Extract base params (handles frequency conversion from series if needed)
@@ -402,6 +463,9 @@ class DFMConfig(BaseModelConfig):
         if config_type == MODEL_TYPE_DDFM:
             return DDFMConfig(**base_params, **DDFMConfig._extract_ddfm(data))
         
+        if config_type == MODEL_TYPE_KDFM:
+            return KDFMConfig(**base_params, **KDFMConfig._extract_kdfm(data))
+        
         # Handle blocks for DFM
         from ...config.adapter import _raise_config_error, _is_dict_like
         blocks_dict = data.get('blocks', {})
@@ -411,7 +475,9 @@ class DFMConfig(BaseModelConfig):
             _raise_config_error(f"blocks must be a dict, got {type(blocks_dict)}")
         
         blocks_dict_normalized = _normalize_blocks_dict(blocks_dict)
-        return DFMConfig(blocks=blocks_dict_normalized, **base_params, **DFMConfig._extract_dfm_params(data))
+        # Pass base_params to _extract_dfm_params to avoid duplicate extraction
+        dfm_params = DFMConfig._extract_dfm_params(data, base_params=base_params)
+        return DFMConfig(blocks=blocks_dict_normalized, **dfm_params)
 
 
 @dataclass
@@ -438,7 +504,7 @@ class DDFMConfig(BaseModelConfig):
     learning_rate: float = 0.001  # Learning rate for Adam optimizer (default: 0.001)
     epochs: int = 100  # Number of training epochs (default: 100)
     batch_size: int = 100  # Batch size for training (default: 100 to match original DDFM)
-    factor_order: int = 1  # VAR lag order for factor dynamics. Must be 1 or 2 (maximum supported order is VAR(2), default: 1)
+    # Note: factor_order removed - factors always use AR(1) dynamics (simplified)
     use_idiosyncratic: bool = True  # Model idio components with AR(1) dynamics (default: True)
     min_obs_idio: int = 5  # Minimum observations for idio AR(1) estimation (default: 5)
     
@@ -456,7 +522,7 @@ class DDFMConfig(BaseModelConfig):
     @classmethod
     def _extract_ddfm(cls, data: Dict[str, Any]) -> Dict[str, Any]:
         """Extract DDFM-specific parameters from config dict."""
-        base_params = cls._extract_base(data)
+        # Don't extract base params here - they're already in base_params from from_dict
         ddfm_params = cls._extract_params(data, {
             'encoder_layers': None,
             'num_factors': None,
@@ -465,7 +531,6 @@ class DDFMConfig(BaseModelConfig):
             'learning_rate': DEFAULT_LEARNING_RATE,
             'epochs': DEFAULT_MAX_EPOCHS,
             'batch_size': DEFAULT_DDFM_BATCH_SIZE,
-            'factor_order': 1,
             'use_idiosyncratic': True,
             'min_obs_idio': DEFAULT_MIN_OBS_IDIO,
             'max_iter': DEFAULT_MAX_MCMC_ITER,
@@ -473,8 +538,7 @@ class DDFMConfig(BaseModelConfig):
             'disp': DEFAULT_DISP,
             'seed': None,
         })
-        base_params.update(ddfm_params)
-        return base_params
+        return ddfm_params
     
     @classmethod
     def from_dict(cls, data: Dict[str, Any]) -> 'DDFMConfig':
@@ -482,7 +546,11 @@ class DDFMConfig(BaseModelConfig):
         result = DFMConfig.from_dict(data)
         if isinstance(result, DDFMConfig):
             return result
-        raise ValueError("Expected DDFMConfig but got DFMConfig")
+        from ..utils.errors import ConfigurationError
+        raise ConfigurationError(
+            "Expected DDFMConfig but got DFMConfig",
+            details=f"Result type: {type(result).__name__}, expected: DDFMConfig"
+        )
 
 
 @dataclass
@@ -491,10 +559,12 @@ class KDFMConfig(BaseModelConfig):
     
     This dataclass contains all configuration parameters for the KDFM model.
     It inherits from BaseModelConfig and adds KDFM-specific parameters.
+    
+    Note: KDFM does not use blocks structure (unlike DFM). Only frequency dict is needed.
     """
     # VARMA parameters
-    ar_order: int = 1  # VAR order p
-    ma_order: int = 0  # MA order q (0 = pure VAR)
+    ar_order: int = DEFAULT_KDFM_AR_ORDER  # VAR order p
+    ma_order: int = DEFAULT_KDFM_MA_ORDER  # MA order q (0 = pure VAR)
     
     # Structural identification
     structural_method: str = 'cholesky'  # 'cholesky', 'full', 'low_rank'
@@ -511,6 +581,42 @@ class KDFMConfig(BaseModelConfig):
     structural_reg_weight: float = DEFAULT_STRUCTURAL_REG_WEIGHT  # Weight for structural loss
     use_regularization: bool = True
     regularization_scale: float = DEFAULT_REGULARIZATION_SCALE
+    
+    # ========================================================================
+    # Factory Methods
+    # ========================================================================
+    
+    @classmethod
+    def _extract_kdfm(cls, data: Dict[str, Any]) -> Dict[str, Any]:
+        """Extract KDFM-specific parameters from config dict."""
+        # Don't extract base params here - they're already in base_params from from_dict
+        kdfm_params = cls._extract_params(data, {
+            'ar_order': DEFAULT_KDFM_AR_ORDER,
+            'ma_order': DEFAULT_KDFM_MA_ORDER,
+            'structural_method': 'cholesky',
+            'structural_rank': None,
+            'learning_rate': DEFAULT_LEARNING_RATE,
+            'max_epochs': DEFAULT_MAX_EPOCHS,
+            'batch_size': DEFAULT_BATCH_SIZE,
+            'weight_decay': DEFAULT_REGULARIZATION_SCALE,
+            'grad_clip_val': DEFAULT_GRAD_CLIP_VAL,
+            'structural_reg_weight': DEFAULT_STRUCTURAL_REG_WEIGHT,
+            'use_regularization': True,
+            'regularization_scale': DEFAULT_REGULARIZATION_SCALE,
+        })
+        return kdfm_params
+    
+    @classmethod
+    def from_dict(cls, data: Dict[str, Any]) -> 'KDFMConfig':
+        """Create KDFMConfig from dictionary (delegates to DFMConfig.from_dict for type detection)."""
+        result = DFMConfig.from_dict(data)
+        if isinstance(result, KDFMConfig):
+            return result
+        from ..utils.errors import ConfigurationError
+        raise ConfigurationError(
+            f"Expected KDFMConfig but got {type(result).__name__}",
+            details=f"Result type: {type(result).__name__}, expected: KDFMConfig"
+        )
 
 
 # ============================================================================
