@@ -16,11 +16,15 @@ import pandas as pd
 from ..config import DFMConfig
 from ..config.constants import (
     FREQUENCY_HIERARCHY, PERIODS_PER_YEAR, DEFAULT_CLOCK_FREQUENCY, 
-    DEFAULT_HIERARCHY_VALUE, HIGH_CORR_THRESHOLD, DEFAULT_START_DATE
+    DEFAULT_HIERARCHY_VALUE, HIGH_CORR_THRESHOLD, DEFAULT_START_DATE,
+    MAX_WARNING_ITEMS
 )
 from ..config import get_periods_per_year
 from ..dataset.process import TimeIndex, parse_timestamp
 from ..logger import get_logger
+from ..utils.common import ensure_numpy
+from ..utils.errors import ConfigurationError, DataValidationError
+from ..utils.helper import get_config_attr
 
 _logger = get_logger(__name__)
 
@@ -32,6 +36,10 @@ class BaseDataModule(ABC):
     - Data reading and loading functionality
     - Common initialization patterns
     - Abstract methods for subclasses to implement
+    - Target scaler handling: Users must provide a **fitted** sklearn scaler 
+      (StandardScaler, RobustScaler, etc.) in config.target_scaler if they want 
+      to scale target series. The scaler must be fitted on target data before 
+      passing to the config. Use scaler.inverse_transform() for unstandardization.
     
     Subclasses:
     - DFMDataModule: Inherits from BaseDataModule only (ABC)
@@ -76,9 +84,10 @@ class BaseDataModule(ABC):
             config = source.load()
         
         if config is None:
-            raise ValueError(
+            raise ConfigurationError(
                 "DataModule initialization failed: either config or config_path must be provided. "
-                "Please provide a DFMConfig object or a path to a configuration file."
+                "Please provide a DFMConfig object or a path to a configuration file.",
+                details="Both config and config_path are None. One must be provided."
             )
         
         self.config = config
@@ -94,7 +103,7 @@ class BaseDataModule(ABC):
             self.target_series = list(target_series)
         
         # Get target_scaler from config
-        self.target_scaler = getattr(config, 'target_scaler', None)
+        self.target_scaler = get_config_attr(config, 'target_scaler', None)
         
         # Handle time_index (consolidate time_index_column into time_index)
         # Support backward compatibility: if time_index_column is in kwargs, use it
@@ -112,9 +121,8 @@ class BaseDataModule(ABC):
             self.time_index = None
             self.time_index_column = None
         
-        # Will be set in setup()
-        self.Mx: Optional[np.ndarray] = None
-        self.Wx: Optional[np.ndarray] = None
+        # Target scaler is stored in self.target_scaler (set in __init__)
+        # Use scaler.inverse_transform() for unstandardization, not Mx/Wx arrays
     
     @staticmethod
     def read_data(datafile: Union[str, Path]) -> Tuple[np.ndarray, TimeIndex, List[str]]:
@@ -156,7 +164,10 @@ class BaseDataModule(ABC):
             # Use pandas read_csv with low_memory=False to infer all columns properly
             df = pd.read_csv(datafile, low_memory=False)
         except (IOError, pd.errors.EmptyDataError, pd.errors.ParserError) as e:
-            raise ValueError(f"Failed to read data file {datafile}: {e}")
+            raise DataValidationError(
+                f"Failed to read data file {datafile}: {e}",
+                details=f"File path: {datafile}. Check file exists and is readable."
+            )
         
         # Check if first column is a date column or metadata
         first_col = df.columns[0]
@@ -244,9 +255,15 @@ class BaseDataModule(ABC):
                     dates = [start_date + timedelta(days=int(df[first_col].iloc[i])) for i in range(n_periods)]
                     Time = TimeIndex(dates)
                 else:
-                    raise ValueError(f"Failed to parse date column '{first_col}': {e}")
+                    raise DataValidationError(
+                        f"Failed to parse date column '{first_col}': {e}",
+                        details=f"Column '{first_col}' could not be parsed as dates. Check date format."
+                    )
             except (ValueError, TypeError) as e2:
-                raise ValueError(f"Failed to parse date column '{first_col}': {e2}")
+                raise DataValidationError(
+                    f"Failed to parse date column '{first_col}': {e2}",
+                    details=f"Column '{first_col}' parsing failed with secondary error. Check date format."
+                )
         
         # Extract series data (all columns except first)
         series_cols = [col for col in df.columns if col != first_col]
@@ -327,7 +344,7 @@ class BaseDataModule(ABC):
             if isinstance(mask, pd.Series):
                 mask = mask.values
             # Ensure mask is boolean numpy array
-            mask = np.asarray(mask, dtype=bool)
+            mask = ensure_numpy(mask, dtype=bool)
             Z = Z.iloc[mask] if isinstance(Z, pd.DataFrame) else Z[mask]
             # TimeIndex has filter() method - convert mask to list for filter()
             Time = Time.filter(mask.tolist())
@@ -340,7 +357,7 @@ class BaseDataModule(ABC):
             if isinstance(mask, pd.Series):
                 mask = mask.values
             # Ensure mask is boolean numpy array
-            mask = np.asarray(mask, dtype=bool)
+            mask = ensure_numpy(mask, dtype=bool)
             Z = Z.iloc[mask] if isinstance(Z, pd.DataFrame) else Z[mask]
             # TimeIndex has filter() method - convert mask to list for filter()
             Time = Time.filter(mask.tolist())
@@ -351,7 +368,7 @@ class BaseDataModule(ABC):
         _logger.info(f"Loaded data: {X.shape[0]} time periods, {X.shape[1]} series (raw, not transformed)")
         
         # Validate data quality
-        clock = getattr(self.config, 'clock', DEFAULT_CLOCK_FREQUENCY)
+        clock = get_config_attr(self.config, 'clock', DEFAULT_CLOCK_FREQUENCY)
         clock_hierarchy = FREQUENCY_HIERARCHY.get(clock, DEFAULT_HIERARCHY_VALUE)
         
         # Get column names from data if available
@@ -372,9 +389,10 @@ class BaseDataModule(ABC):
             
             series_hierarchy = FREQUENCY_HIERARCHY.get(freq, DEFAULT_HIERARCHY_VALUE)
             if series_hierarchy < clock_hierarchy:
-                raise ValueError(
+                raise DataValidationError(
                     f"Series '{series_ids[i]}' has frequency '{freq}' which is faster than clock '{clock}'. "
-                    f"Higher frequencies (daily, weekly) are not supported."
+                    f"Higher frequencies (daily, weekly) are not supported.",
+                    details=f"Series frequency hierarchy ({series_hierarchy}) < clock hierarchy ({clock_hierarchy})"
                 )
             
             # Check for T < N condition (may cause numerical issues)
@@ -383,13 +401,11 @@ class BaseDataModule(ABC):
                 warnings_list.append((series_ids[i], valid_obs, X.shape[1]))
         
         if len(warnings_list) > 0:
-            from ..config.constants import MAX_WARNING_ITEMS
             for series_id, T_obs, N_total in warnings_list[:MAX_WARNING_ITEMS]:
                 _logger.warning(
                     f"Series '{series_id}': T={T_obs} < N={N_total} (may cause numerical issues). "
                     f"Suggested fix: increase sample size or reduce number of series."
                 )
-            from ..config.constants import MAX_WARNING_ITEMS
             if len(warnings_list) > MAX_WARNING_ITEMS:
                 _logger.warning(f"... and {len(warnings_list) - MAX_WARNING_ITEMS} more series with T < N")
             
@@ -412,14 +428,13 @@ class BaseDataModule(ABC):
                 extreme_missing_series.append((series_id, ratio))
         
         if len(extreme_missing_series) > 0:
-            from ..config.constants import MAX_WARNING_ITEMS
             for series_id, ratio in extreme_missing_series[:MAX_WARNING_ITEMS]:
                 _logger.warning(
                     f"Series '{series_id}' has {ratio:.1%} missing data (>90%). "
                     f"This may cause estimation issues. Consider removing this series or increasing data coverage."
                 )
-            if len(extreme_missing_series) > 5:
-                _logger.warning(f"... and {len(extreme_missing_series) - 5} more series with >90% missing data")
+            if len(extreme_missing_series) > MAX_WARNING_ITEMS:
+                _logger.warning(f"... and {len(extreme_missing_series) - MAX_WARNING_ITEMS} more series with >90% missing data")
             
             warnings.warn(
                 f"Extreme missing data detected: {len(extreme_missing_series)} series have >90% missing values. "
@@ -445,9 +460,10 @@ class BaseDataModule(ABC):
             If setup() has not been called
         """
         if not hasattr(self, 'data_processed') or self.data_processed is None:
-            raise RuntimeError(
+            raise ConfigurationError(
                 f"DataModule {method_name} failed: setup() must be called first. "
-                f"Please call dm.setup() before calling {method_name}()."
+                f"Please call dm.setup() before calling {method_name}().",
+                details=f"Method '{method_name}' requires data_processed attribute which is set by setup()."
             )
     
     def _is_mixed_frequency(self, columns: Optional[List[str]] = None) -> bool:
@@ -463,7 +479,7 @@ class BaseDataModule(ABC):
         bool
             True if any series has frequency slower than clock, False otherwise
         """
-        clock = getattr(self.config, 'clock', DEFAULT_CLOCK_FREQUENCY)
+        clock = get_config_attr(self.config, 'clock', DEFAULT_CLOCK_FREQUENCY)
         clock_hierarchy = FREQUENCY_HIERARCHY.get(clock, DEFAULT_HIERARCHY_VALUE)
         
         frequencies = self.config.get_frequencies(columns)
@@ -477,6 +493,21 @@ class BaseDataModule(ABC):
         
         return False
     
+    def _filter_numeric_columns(self, df: pd.DataFrame) -> pd.DataFrame:
+        """Filter DataFrame to only numeric columns.
+        
+        Parameters
+        ----------
+        df : pd.DataFrame
+            DataFrame to filter
+            
+        Returns
+        -------
+        pd.DataFrame
+            DataFrame with only numeric columns
+        """
+        return df.select_dtypes(include=[np.number])
+    
     def _extract_time_index_from_dataframe(self, df: pd.DataFrame) -> TimeIndex:
         """Extract time index from DataFrame using time_index_column.
         
@@ -491,15 +522,19 @@ class BaseDataModule(ABC):
             Extracted time index
         """
         if self.time_index_column is None:
-            raise ValueError("time_index_column must be set to extract time index from DataFrame")
+            raise ConfigurationError(
+                "time_index_column must be set to extract time index from DataFrame",
+                details="time_index_column attribute is None. Set it before calling extract_time_index()."
+            )
         
         time_cols = [self.time_index_column] if isinstance(self.time_index_column, str) else self.time_index_column
         
         missing_cols = [col for col in time_cols if col not in df.columns]
         if missing_cols:
-            raise ValueError(
+            raise DataValidationError(
                 f"time_index_column(s) {missing_cols} not found in DataFrame. "
-                f"Available columns: {list(df.columns)}"
+                f"Available columns: {list(df.columns)}",
+                details=f"Requested columns: {missing_cols}. DataFrame has {len(df.columns)} columns."
             )
         
         time_data = df[time_cols]
@@ -511,35 +546,6 @@ class BaseDataModule(ABC):
         
         return TimeIndex(time_list)
     
-    def _compute_target_mx_wx(self, df: pd.DataFrame, target_cols: List[str]) -> Tuple[Optional[np.ndarray], Optional[np.ndarray]]:
-        """Compute target Mx, Wx statistics from scaler.
-        
-        Parameters
-        ----------
-        df : pd.DataFrame
-            DataFrame containing target columns
-        target_cols : List[str]
-            List of target column names
-            
-        Returns
-        -------
-        Tuple[Optional[np.ndarray], Optional[np.ndarray]]
-            (Mx, Wx) arrays or (None, None) if no scaler
-        """
-        if not target_cols:
-            return np.array([]), np.array([])
-        
-        target_values = np.asarray(df[target_cols].values)
-        
-        if self.target_scaler is not None:
-            from ..dataset.process import _extract_mx_wx
-            mx, wx = _extract_mx_wx(self.target_scaler, target_values)
-        else:
-            mx, wx = None, None
-        
-        # Set defaults if extraction failed
-        mx, wx = self._set_default_mx_wx(mx, wx, len(target_cols))
-        return mx, wx
     
     def _sort_data_by_config(
         self,
