@@ -14,6 +14,14 @@ from typing import TYPE_CHECKING, Any, Dict, List, Optional, Tuple, Union
 import numpy as np
 from scipy.linalg import block_diag
 
+if TYPE_CHECKING:
+    from torch import Tensor
+else:
+    try:
+        from torch import Tensor
+    except ImportError:
+        Tensor = Any
+
 # NumPy-based Kalman filter (pykalman) - now a required dependency
 from ..ssm.kalman import DFMKalmanFilter
 
@@ -250,11 +258,13 @@ class DFM(BaseFactorModel):
         ModelNotInitializedError
             If parameters are not initialized
         """
-        check_condition(
-            not any(p is None for p in [self.A, self.C, self.Q, self.R, self.Z_0, self.V_0]),
-            ModelNotInitializedError,
-            f"{self.__class__.__name__}: Model parameters not initialized",
-            details="Parameters (A, C, Q, R, Z_0, V_0) are required but are None. Please call fit() first to initialize parameters"
+        from ..numeric.validator import validate_parameters_initialized
+        validate_parameters_initialized(
+            {
+                'A': self.A, 'C': self.C, 'Q': self.Q,
+                'R': self.R, 'Z_0': self.Z_0, 'V_0': self.V_0
+            },
+            model_name=self.__class__.__name__
         )
     
     def _update_parameters(self, A: np.ndarray, C: np.ndarray, Q: np.ndarray,
@@ -266,19 +276,14 @@ class DFM(BaseFactorModel):
         A, C, Q, R, Z_0, V_0 : np.ndarray
             Parameter arrays
         """
-        # Only convert dtype if necessary (avoid redundant conversions)
-        def _ensure_dtype(arr: Optional[np.ndarray]) -> Optional[np.ndarray]:
-            # Conditional logic: Return None if input is None (not validation)
-            if arr is None:
-                return None
-            return arr.astype(DEFAULT_DTYPE) if arr.dtype != DEFAULT_DTYPE else arr
-        
-        self.A = _ensure_dtype(A)
-        self.C = _ensure_dtype(C)
-        self.Q = _ensure_dtype(Q)
-        self.R = _ensure_dtype(R)
-        self.Z_0 = _ensure_dtype(Z_0)
-        self.V_0 = _ensure_dtype(V_0)
+        from ..utils.common import ensure_numpy
+        # Convert to numpy and ensure dtype (only if necessary)
+        self.A = ensure_numpy(A, dtype=DEFAULT_DTYPE) if A is not None else None
+        self.C = ensure_numpy(C, dtype=DEFAULT_DTYPE) if C is not None else None
+        self.Q = ensure_numpy(Q, dtype=DEFAULT_DTYPE) if Q is not None else None
+        self.R = ensure_numpy(R, dtype=DEFAULT_DTYPE) if R is not None else None
+        self.Z_0 = ensure_numpy(Z_0, dtype=DEFAULT_DTYPE) if Z_0 is not None else None
+        self.V_0 = ensure_numpy(V_0, dtype=DEFAULT_DTYPE) if V_0 is not None else None
     
     def _initialize_clock_freq_idio(
         self,
@@ -1062,6 +1067,39 @@ class DFM(BaseFactorModel):
         
         return self.training_state
     
+    def _create_kalman_filter(
+        self,
+        initial_state_mean: Optional[np.ndarray] = None,
+        initial_state_covariance: Optional[np.ndarray] = None
+    ) -> DFMKalmanFilter:
+        """Create Kalman filter with current training state parameters.
+        
+        Parameters
+        ----------
+        initial_state_mean : np.ndarray, optional
+            Initial state mean. If None, uses training_state.Z_0
+        initial_state_covariance : np.ndarray, optional
+            Initial state covariance. If None, uses training_state.V_0
+            
+        Returns
+        -------
+        DFMKalmanFilter
+            Configured Kalman filter instance
+        """
+        if initial_state_mean is None:
+            initial_state_mean = self.training_state.Z_0
+        if initial_state_covariance is None:
+            initial_state_covariance = self.training_state.V_0
+        
+        return DFMKalmanFilter(
+            transition_matrices=self.training_state.A,
+            observation_matrices=self.training_state.C,
+            transition_covariance=self.training_state.Q,
+            observation_covariance=self.training_state.R,
+            initial_state_mean=initial_state_mean,
+            initial_state_covariance=initial_state_covariance
+        )
+    
     def _compute_smoothed_factors(self) -> np.ndarray:
         """Compute smoothed factors using Kalman filter.
         
@@ -1077,15 +1115,7 @@ class DFM(BaseFactorModel):
             details="Please call fit() method before computing smoothed factors"
         )
         
-        kalman_final = DFMKalmanFilter(
-            transition_matrices=self.training_state.A,
-            observation_matrices=self.training_state.C,
-            transition_covariance=self.training_state.Q,
-            observation_covariance=self.training_state.R,
-            initial_state_mean=self.training_state.Z_0,
-            initial_state_covariance=self.training_state.V_0
-        )
-        
+        kalman_final = self._create_kalman_filter()
         y_masked = np.ma.masked_invalid(self.data_processed)
         smoothed_state_means, _ = kalman_final.smooth(y_masked)
         return smoothed_state_means
@@ -1125,6 +1155,90 @@ class DFM(BaseFactorModel):
         )
     
     
+    def update(self, data: Union[np.ndarray, Any]) -> None:
+        """Update model state with new observations via Kalman filtering/smoothing.
+        
+        This method runs Kalman filtering/smoothing on new data to update the
+        latent factors, but keeps model parameters (A, C, Q, R) fixed.
+        
+        After calling update(), the model's internal state (result.Z and data_processed)
+        is extended with the new observations. Subsequent calls to predict() will use
+        the updated state.
+        
+        **Data Shape**: The input data must be 2D with shape (T_new x N) where:
+        - T_new: Number of new time steps (can be any positive integer)
+        - N: Number of series (must match training data)
+        
+        **Supported Types**:
+        - numpy.ndarray: (T_new x N) array
+        - pandas.DataFrame: DataFrame with N columns, T_new rows
+        - polars.DataFrame: DataFrame with N columns, T_new rows
+        
+        **Important**: Data must be preprocessed by the user (same preprocessing as training).
+        Only target scaler is handled internally if needed.
+        
+        Parameters
+        ----------
+        data : np.ndarray, pandas.DataFrame, or polars.DataFrame
+            New preprocessed observations with shape (T_new x N) where:
+            - T_new: Number of new time steps (any positive integer)
+            - N: Number of series (must match training data)
+            Data must be preprocessed by user (same preprocessing as training).
+            
+        Notes
+        -----
+        - This updates factors via filtering/smoothing, NOT parameter retraining
+        - For parameter retraining, use fit() with concatenated data
+        - After update(), predict() will use the updated factor state
+        - New data must have same number of series (N) as training data
+        - User must preprocess data themselves (same preprocessing as training)
+        
+        Raises
+        ------
+        ModelNotTrainedError
+            If model has not been trained yet
+        DataValidationError
+            If data shape doesn't match training data
+        """
+        # Validate and convert data (no preprocessing - user must preprocess)
+        from ..numeric.validator import validate_and_convert_update_data
+        data_new = validate_and_convert_update_data(
+            data, 
+            self.data_processed, 
+            dtype=DEFAULT_DTYPE,
+            model_name=self.__class__.__name__
+        )
+        
+        # Get current result (compute if needed)
+        result = self._ensure_result()
+        
+        # Get last smoothed state from training as initial state for new data
+        Z_last = result.Z[-1, :] if result.Z.shape[0] > 0 else result.Z_0
+        V_last = result.V_0  # Use original V_0 or could compute from last state covariance
+        
+        # Create Kalman filter with current parameters and new initial state
+        kalman_new = self._create_kalman_filter(
+            initial_state_mean=Z_last,
+            initial_state_covariance=V_last
+        )
+        
+        # Run filter and smooth on new data
+        y_masked = np.ma.masked_invalid(data_new)
+        Z_new, V_smooth_new, _, _ = kalman_new.filter_and_smooth(y_masked)
+        
+        # Update model state: append new factors and data
+        # Concatenate new factors to existing result.Z
+        result.Z = np.vstack([result.Z, Z_new])
+        
+        # Append new data to data_processed
+        self.data_processed = np.vstack([self.data_processed, data_new])
+        
+        # Update smoothed data (x_sm) in result
+        result.x_sm = result.Z @ result.C.T
+        
+        # Invalidate cached result (or it's already updated above)
+        # The result object is updated in place, so _result is still valid
+    
     def load_config(
         self,
         source: Optional[Union[str, Path, Dict[str, Any], DFMConfig, ConfigSource]] = None,
@@ -1161,6 +1275,7 @@ class DFM(BaseFactorModel):
         self,
         horizon: Optional[int] = None,
         *,
+        data: Optional[Union[np.ndarray, Tensor, Any]] = None,
         return_series: bool = True,
         return_factors: bool = True
     ) -> Union[np.ndarray, Tuple[np.ndarray, np.ndarray]]:
@@ -1172,11 +1287,25 @@ class DFM(BaseFactorModel):
         Target series are determined from the DataModule's target_series attribute,
         which should be set during DataModule initialization.
         
+        **New Data Initialization**: If `data` is provided, the method runs a
+        Kalman filter forward pass on the new data to compute the initial factor
+        state for forecasting. This does NOT modify the model's internal state.
+        Use `update()` method if you want to update the model state with new data.
+        
+        **Important**: If `data` is provided, it must be preprocessed by the user
+        (same preprocessing as training). Only target scaler is handled internally.
+        
         Parameters
         ----------
         horizon : int, optional
             Number of periods ahead to forecast. If None, defaults to 1 year
             of periods based on clock frequency.
+        data : np.ndarray, torch.Tensor, pandas.DataFrame, or polars.DataFrame, optional
+            New preprocessed observations to use for initializing forecast. If provided,
+            runs Kalman filter forward pass to compute Z_last from the new data.
+            Does NOT modify model state (use update() for that).
+            Must have shape (T_new x N) where N matches training data.
+            Data must be preprocessed by user (same preprocessing as training).
         return_series : bool, optional
             Whether to return forecasted series (default: True)
         return_factors : bool, optional
@@ -1196,7 +1325,12 @@ class DFM(BaseFactorModel):
         ------
         ValueError
             If DataModule has no target_series set
+        ModelNotTrainedError
+            If model has not been trained yet
+        DataValidationError
+            If data shape doesn't match training data
         """
+        # Validate model is trained
         check_condition(
             self.training_state is not None,
             ModelNotTrainedError,
@@ -1207,11 +1341,8 @@ class DFM(BaseFactorModel):
         # Validate parameters are initialized
         self._check_parameters_initialized()
         
-        # Conditional logic: Generate result if not already computed (not validation)
-        if self._result is None:
-            self._result = self.get_result()
-        
-        result = self._result
+        # Get result (compute if needed)
+        result = self._ensure_result()
         
         check_condition(
             result.Z is not None,
@@ -1220,17 +1351,11 @@ class DFM(BaseFactorModel):
             details="This may indicate the model was not properly trained or result object is corrupted"
         )
         
-        # Conditional logic: Compute default horizon if not provided (not validation)
+        # Compute and validate horizon
         if horizon is None:
             horizon = self._compute_default_horizon()
-        
-        # Validate horizon
-        check_condition(
-            horizon > 0,
-            PredictionError,
-            f"horizon must be positive, got {horizon}",
-            details="Forecast horizon must be a positive integer"
-        )
+        from ..numeric.validator import validate_horizon
+        horizon = validate_horizon(horizon)
         
         # Extract model parameters
         A = result.A
@@ -1238,44 +1363,57 @@ class DFM(BaseFactorModel):
         target_scaler = result.target_scaler  # Use scaler object instead of Mx/Wx
         p = result.p  # VAR order (always available after training)
         
-        # Use training state for initial factor state
-        # For DFM, we use the last smoothed state from training
-        # History-based updates can be added later if needed
-        Z_last = result.Z[-1, :] if result.Z.shape[0] > 0 else np.zeros(result.A.shape[0], dtype=DEFAULT_DTYPE)
+        # Determine initial factor state
+        if data is not None:
+            # Use new data to compute initial factor state via Kalman filter forward pass
+            # This does NOT modify model state
+            # User must preprocess data themselves (same preprocessing as training)
+            from ..numeric.validator import validate_and_convert_update_data
+            data_new = validate_and_convert_update_data(
+                data,
+                self.data_processed,
+                dtype=DEFAULT_DTYPE,
+                model_name=self.__class__.__name__
+            )
+            
+            # Get last smoothed state from training as initial state for filtering
+            Z_initial = result.Z[-1, :] if result.Z.shape[0] > 0 else result.Z_0
+            V_initial = result.V_0
+            
+            # Create Kalman filter with current parameters
+            kalman_filter = self._create_kalman_filter(
+                initial_state_mean=Z_initial,
+                initial_state_covariance=V_initial
+            )
+            
+            # Run filter (forward pass only, not smooth) on new data
+            y_masked = np.ma.masked_invalid(data_new)
+            filtered_states, _ = kalman_filter.filter(y_masked)
+            
+            # Extract last filtered state as initial state for forecasting
+            Z_last = filtered_states[-1, :] if filtered_states.shape[0] > 0 else Z_initial
+        else:
+            # Use training state for initial factor state
+            # For DFM, we use the last smoothed state from training
+            Z_last = result.Z[-1, :] if result.Z.shape[0] > 0 else np.zeros(result.A.shape[0], dtype=DEFAULT_DTYPE)
         
-        # Validate factor state
-        check_condition(
-            not np.any(np.isnan(Z_last)),
-            NumericalError,
-            f"DFM prediction failed: {np.sum(np.isnan(Z_last))}/{len(Z_last)} factors contain NaN ({np.sum(np.isnan(Z_last)) / len(Z_last):.1%})",
-            details="Model may not have converged. Try increasing max_iter or checking data quality"
-        )
-        
-        # Validate parameters are finite
-        check_condition(
-            np.all(np.isfinite(A)) and np.all(np.isfinite(C)),
-            NumericalError,
-            "DFM prediction failed: model parameters (A or C) contain NaN/Inf",
-            details="Check training convergence and data quality"
-        )
+        # Validate factor state and parameters are finite
+        from ..numeric.validator import validate_no_nan_inf
+        validate_no_nan_inf(Z_last, name="factor state Z_last")
+        validate_no_nan_inf(A, name="transition matrix A")
+        validate_no_nan_inf(C, name="observation matrix C")
         
         # Resolve target series from DataModule (target_series should be set at initialization)
+        # _resolve_target_series() already validates and raises DataError if none found
         series_ids = self._config.get_series_ids() if self._config is not None else result.series_ids
         target_series, target_indices = self._resolve_target_series(series_ids, result)
         
-        check_condition(
-            target_series is not None and len(target_series) > 0,
-            ValueError,
-            "DFM prediction failed: no target_series found in DataModule. "
-            "Please set target_series when creating the DataModule (e.g., DFMDataModule(..., target_series=['series_id']))."
-        )
-        
-        check_condition(
-            target_indices is not None and len(target_indices) > 0,
-            DataError,
-            f"DFM prediction failed: none of the target series found in available series",
-            details=f"Target: {target_series}, Available: {series_ids}"
-        )
+        # Additional validation: ensure target_series was set in DataModule
+        if target_series is None or len(target_series) == 0:
+            raise ValueError(
+                "DFM prediction failed: no target_series found in DataModule. "
+                "Please set target_series when creating the DataModule (e.g., DFMDataModule(..., target_series=['series_id']))."
+            )
         
         # Forecast factors using VAR dynamics (common helper)
         Z_prev = result.Z[-2, :] if result.Z.shape[0] >= 2 and p == 2 else None
@@ -1302,18 +1440,10 @@ class DFM(BaseFactorModel):
             # No scaler - assume already in original scale
             X_forecast = X_forecast_std
         
-        # Validate forecast results are finite
-        nan_count = np.sum(~np.isfinite(X_forecast))
-        check_condition(
-            nan_count == 0,
-            NumericalError,
-            f"DFM prediction failed: produced {nan_count} NaN/Inf values in forecast",
-            details="Possible numerical instability. Check model parameters, training convergence, and data quality."
-        )
-        
-        # Ensure X_forecast is numpy array (handles torch inputs if present)
+        # Ensure X_forecast is numpy array and validate it's finite
         from ..utils.common import ensure_numpy
         X_forecast = ensure_numpy(X_forecast, dtype=DEFAULT_DTYPE)
+        validate_no_nan_inf(X_forecast, name="forecast X_forecast")
         
         # Validate forecast values are within reasonable bounds (only if scaler available)
         # Note: Validation using std devs requires scaler attributes, skip if not available
@@ -1344,8 +1474,7 @@ class DFM(BaseFactorModel):
                 pass
         
         if return_factors:
-            from ..utils.helper import validate_finite_array, get_config_attr
-            validate_finite_array(Z_forecast, "factor forecast", context="DFM prediction")
+            validate_no_nan_inf(Z_forecast, name="factor forecast Z_forecast")
         
         if return_series and return_factors:
             return X_forecast, Z_forecast
@@ -1359,21 +1488,13 @@ class DFM(BaseFactorModel):
         
         Raises
         ------
-        ValueError
+        ModelNotTrainedError
             If model has not been trained yet
         """
-        # Check if trained and extract result from training state if needed
-        self._check_trained()
-        if self._result is None:
-            # Generate result from training state if not already computed
-            self._result = self.get_result()
-        check_condition(
-            isinstance(self._result, DFMResult),
-            ModelNotTrainedError,
-            f"Expected DFMResult but got {type(self._result)}",
-            details="Model result type mismatch. Please ensure model was properly trained"
-        )
-        return self._result
+        result = self._ensure_result()
+        # Type assertion: get_result() always returns DFMResult for DFM model
+        assert isinstance(result, DFMResult), f"Expected DFMResult but got {type(result)}"
+        return result
     
     
     
