@@ -560,6 +560,17 @@ class KDFM(BaseFactorModel, pl.LightningModule):
         # Forward pass
         y_pred = self.forward(data)
         
+        # Store processed data for shape validation in update() (store first batch)
+        if self.data_processed is None:
+            from ..utils.common import ensure_numpy
+            # Store as numpy array in base class format
+            if data.ndim == 3:
+                # Batch format: take first sample
+                data_sample = data[0]
+            else:
+                data_sample = data
+            self.data_processed = ensure_numpy(data_sample, dtype=DEFAULT_DTYPE)
+        
         # Compute losses using training step handler
         total_loss = self._compute_training_loss(y_pred, target, device)
         
@@ -972,8 +983,8 @@ class KDFM(BaseFactorModel, pl.LightningModule):
             details="companion_ar is None"
         )
         
-        # Get result for parameters
-        result = self.get_result()
+        # Get result for parameters (compute if needed)
+        result = self._ensure_result()
         
         # Extract n_vars using helper method with fallback hierarchy
         n_vars = self._extract_n_vars_from_sources(
@@ -1088,15 +1099,9 @@ class KDFM(BaseFactorModel, pl.LightningModule):
                     z_t = structural_shocks.squeeze(0)  # (K,)
                 
                 Z_last = ensure_numpy(z_t)
-                if np.any(np.isnan(Z_last)) or np.any(np.isinf(Z_last)):
-                    raise NumericalError(
-                        f"KDFM predict: Factor state Z_last contains NaN/Inf values. "
-                        f"Shape: {Z_last.shape}, NaN count: {np.sum(np.isnan(Z_last))}, "
-                        f"Inf count: {np.sum(np.isinf(Z_last))}. "
-                        f"This may indicate: (1) numerical instability in companion matrix operations, "
-                        f"(2) invalid input data in last_observation, or (3) model not properly trained. "
-                        f"Please check: model training convergence, input data validity, and companion matrix stability."
-                    )
+                # Validate factor state is finite
+                from ..numeric.validator import validate_no_nan_inf
+                validate_no_nan_inf(Z_last, name="factor state Z_last")
             except COMPUTATION_ERROR_TYPES as e:
                 # Catch computation errors during factor state extraction from observation
                 # Note: NumericalError/PredictionError inherit from Exception, not the caught types above,
@@ -1148,13 +1153,8 @@ class KDFM(BaseFactorModel, pl.LightningModule):
         state[:K] = Z_last.astype(DEFAULT_DTYPE)
         
         # Validate state initialization
-        if np.any(np.isnan(state)) or np.any(np.isinf(state)):
-            raise NumericalError(
-                f"KDFM predict: Initial state vector contains NaN/Inf values. "
-                f"State shape: {state.shape}, NaN count: {np.sum(np.isnan(state))}, "
-                f"Inf count: {np.sum(np.isinf(state))}. "
-                f"This indicates invalid factor state Z_last. Please verify last_observation is valid."
-            )
+        from ..numeric.validator import validate_no_nan_inf
+        validate_no_nan_inf(state, name="initial state vector")
         
         # Generate forecasts using companion matrix powers
         # For VAR(1): Z_{t+h} = A^h @ Z_t
@@ -1163,25 +1163,25 @@ class KDFM(BaseFactorModel, pl.LightningModule):
         for h in range(horizon):
             Z_forecast[h, :] = state[:K].copy()
             state = A_np @ state
-            if np.any(np.isnan(state)) or np.any(np.isinf(state)):
+            # Validate state is finite at each step
+            try:
+                validate_no_nan_inf(state, name=f"forecast state at horizon {h+1}")
+            except NumericalError as e:
                 raise PredictionError(
                     f"KDFM predict: Factor forecast generation produced NaN/Inf at horizon {h+1}. "
-                    f"State shape: {state.shape}, NaN count: {np.sum(np.isnan(state))}, "
-                    f"Inf count: {np.sum(np.isinf(state))}. "
                     f"This indicates numerical instability in companion matrix multiplication. "
                     f"Possible causes: (1) Companion matrix A has eigenvalues >= DEFAULT_EIGENVALUE_MAX_MAGNITUDE (1.0) (unstable), "
                     f"(2) Large forecast horizon causing numerical overflow, (3) Invalid initial state. "
                     f"Please check: companion matrix stability (max eigenvalue < DEFAULT_EIGENVALUE_MAX_MAGNITUDE (1.0)), "
                     f"forecast horizon (try smaller horizon), and initial factor state validity."
-                )
+                ) from e
         
         # Transform factors to observations
         X_forecast = Z_forecast @ result.C.T  # (horizon, K) @ (K, N) -> (horizon, N)
         
-        # Validate forecasts
-        from ..utils.helper import validate_finite_array
-        validate_finite_array(Z_forecast, "factor forecast Z_forecast", context="KDFM prediction")
-        validate_finite_array(X_forecast, "final forecast X_forecast", context="KDFM prediction")
+        # Validate forecasts are finite
+        validate_no_nan_inf(Z_forecast, name="factor forecast Z_forecast")
+        validate_no_nan_inf(X_forecast, name="final forecast X_forecast")
         
         # Return based on flags
         if return_series and return_factors:
@@ -1715,6 +1715,20 @@ class KDFM(BaseFactorModel, pl.LightningModule):
         
         return result
     
+    @property
+    def result(self) -> KDFMResult:
+        """Get model result from training state.
+        
+        Raises
+        ------
+        ModelNotTrainedError
+            If model has not been trained yet
+        """
+        result = self._ensure_result()
+        # Type assertion: get_result() always returns KDFMResult for KDFM model
+        assert isinstance(result, KDFMResult), f"Expected KDFMResult but got {type(result)}"
+        return result
+    
     def _compute_irfs_from_params(
         self,
         ar_transition: Optional[np.ndarray],
@@ -1852,4 +1866,103 @@ class KDFM(BaseFactorModel, pl.LightningModule):
                 f"KDFM IRF computation failed: {type(e).__name__} during IRF computation: {e}",
                 details="This may occur due to memory constraints, invalid indices, or system errors. Please check system resources and input parameters."
             ) from e
+    
+    def update(self, data: Union[np.ndarray, Any]) -> None:
+        """Update model state with new observations via companion matrix forward pass.
+        
+        This method runs the KDFM forward pass on new data to update the latent factors,
+        but keeps model parameters fixed.
+        
+        After calling update(), the model's internal state (result.Z and data_processed)
+        is extended with the new observations. Subsequent calls to predict() will use
+        the updated state.
+        
+        **Data Shape**: The input data must be 2D with shape (T_new x N) where:
+        - T_new: Number of new time steps (can be any positive integer)
+        - N: Number of series (must match training data)
+        
+        **Supported Types**:
+        - numpy.ndarray: (T_new x N) array
+        - pandas.DataFrame: DataFrame with N columns, T_new rows
+        - polars.DataFrame: DataFrame with N columns, T_new rows
+        
+        **Important**: Data must be preprocessed by the user (same preprocessing as training).
+        Only target scaler is handled internally if needed.
+        
+        Parameters
+        ----------
+        data : np.ndarray, pandas.DataFrame, or polars.DataFrame
+            New preprocessed observations with shape (T_new x N) where:
+            - T_new: Number of new time steps (any positive integer)
+            - N: Number of series (must match training data)
+            Data must be preprocessed by user (same preprocessing as training).
+            
+        Notes
+        -----
+        - This updates factors via companion matrix forward pass, NOT parameter retraining
+        - For parameter retraining, use trainer.fit() with concatenated data
+        - After update(), predict() will use the updated factor state
+        - New data must have same number of series (N) as training data
+        - User must preprocess data themselves (same preprocessing as training)
+        
+        Raises
+        ------
+        ModelNotTrainedError
+            If model has not been trained yet
+        DataValidationError
+            If data shape doesn't match training data
+        ModelNotInitializedError
+            If model components are not initialized
+        """
+        from ..utils.common import ensure_tensor, ensure_numpy
+        
+        # Validate and convert data (no preprocessing - user must preprocess)
+        from ..numeric.validator import validate_and_convert_update_data
+        data_new = validate_and_convert_update_data(
+            data,
+            self.data_processed,
+            dtype=DEFAULT_DTYPE,
+            model_name=self.__class__.__name__
+        )
+        
+        # Convert to tensor
+        device = next(self.parameters()).device if self.companion_ar is not None else torch.device('cpu')
+        data_tensor = ensure_tensor(data_new, dtype=DEFAULT_TORCH_DTYPE)
+        data_tensor = data_tensor.to(device)
+        
+        # Check if model is initialized
+        check_condition(
+            self.companion_ar is not None,
+            ModelNotInitializedError,
+            "KDFM model components are not initialized. "
+            "This may occur if initialize_from_data() was not called during training.",
+            details="companion_ar is None"
+        )
+        
+        # Run forward pass to get factors
+        with torch.no_grad():
+            factors_new = self.forward(data_tensor)  # (T_new x K)
+        
+        # Convert to numpy
+        factors_new_np = ensure_numpy(factors_new, dtype=DEFAULT_DTYPE)
+        
+        # Get current result (compute if needed)
+        result = self._ensure_result()
+        
+        # Update model state: append new factors and data
+        result.Z = np.vstack([result.Z, factors_new_np])
+        
+        # Ensure data_processed is numpy array
+        if isinstance(self.data_processed, torch.Tensor):
+            self.data_processed = ensure_numpy(self.data_processed, dtype=DEFAULT_DTYPE)
+        
+        if self.data_processed is not None:
+            self.data_processed = np.vstack([self.data_processed, data_new])
+        else:
+            # If not stored during training, initialize with new data
+            self.data_processed = data_new
+        
+        # Update smoothed data (x_sm) in result
+        result.x_sm = result.Z @ result.C.T
+    
 
