@@ -84,14 +84,22 @@ def ensure_tensor(
 
 
 def ensure_numpy(
-    data: Union[np.ndarray, Tensor, List, float, int],
+    data: Union[np.ndarray, Tensor, List, float, int, Any],
     dtype: Optional[np.dtype] = None
 ) -> np.ndarray:
     """Convert input to numpy array with optional dtype conversion.
     
+    Supports multiple input types:
+    - numpy.ndarray
+    - torch.Tensor
+    - pandas.DataFrame
+    - polars.DataFrame
+    - list, tuple, Sequence
+    - int, float (scalars)
+    
     Parameters
     ----------
-    data : array-like, Tensor, or scalar
+    data : array-like, Tensor, DataFrame, or scalar
         Input data to convert to numpy array
     dtype : np.dtype, optional
         Target dtype (e.g., np.float32). If None, infers from data.
@@ -107,19 +115,82 @@ def ensure_numpy(
     >>> tensor = torch.tensor([1.0, 2.0, 3.0])
     >>> array = ensure_numpy(tensor, dtype=np.float32)
     >>> assert isinstance(array, np.ndarray)
+    
+    >>> import pandas as pd
+    >>> df = pd.DataFrame({"A": [1, 2], "B": [3, 4]})
+    >>> array = ensure_numpy(df)
+    >>> assert isinstance(array, np.ndarray)
     """
     if isinstance(data, np.ndarray):
         array = data
     elif isinstance(data, Tensor):
         array = data.detach().cpu().numpy()
-    elif isinstance(data, Sequence) and not isinstance(data, str):
+    elif isinstance(data, str):
+        # Strings are sequences but should not be converted to numpy arrays
+        raise DataValidationError(
+            f"Cannot convert {type(data).__name__} to numpy array. "
+            f"Supported types: np.ndarray, Tensor, pandas.DataFrame, polars.DataFrame, list, tuple, int, float.",
+            details=f"Input type: {type(data).__name__}, value: {data}"
+        )
+    elif hasattr(data, '__class__'):
+        # Try pandas DataFrame
+        try:
+            import pandas as pd
+            if isinstance(data, pd.DataFrame):
+                array = data.to_numpy()
+            elif data.__class__.__module__ == 'pandas.core.frame':
+                # Fallback for pandas DataFrame detection
+                array = data.to_numpy()
+            else:
+                # Try polars DataFrame
+                try:
+                    import polars as pl
+                    if isinstance(data, pl.DataFrame):
+                        array = data.to_numpy()
+                    else:
+                        raise DataValidationError(
+                            f"Cannot convert {type(data).__name__} to numpy array. "
+                            f"Supported types: np.ndarray, Tensor, pandas.DataFrame, polars.DataFrame, list, tuple, int, float.",
+                            details=f"Input type: {type(data).__name__}, module: {data.__class__.__module__}"
+                        )
+                except (ImportError, AttributeError):
+                    # Not polars, try generic conversion
+                    try:
+                        array = np.array(data)
+                    except Exception as e:
+                        raise DataValidationError(
+                            f"Cannot convert {type(data).__name__} to numpy array: {e}",
+                            details=f"Input type: {type(data).__name__}, module: {data.__class__.__module__}"
+                        ) from e
+        except ImportError:
+            # pandas not available, try polars or generic conversion
+            try:
+                import polars as pl
+                if isinstance(data, pl.DataFrame):
+                    array = data.to_numpy()
+                else:
+                    raise DataValidationError(
+                        f"Cannot convert {type(data).__name__} to numpy array. "
+                        f"pandas not available and input is not polars.DataFrame.",
+                        details=f"Input type: {type(data).__name__}"
+                    )
+            except (ImportError, AttributeError):
+                # Try generic conversion
+                try:
+                    array = np.array(data)
+                except Exception as e:
+                    raise DataValidationError(
+                        f"Cannot convert {type(data).__name__} to numpy array: {e}",
+                        details=f"Input type: {type(data).__name__}"
+                    ) from e
+    elif isinstance(data, Sequence):
         array = np.array(data)
     elif isinstance(data, (int, float)):
         array = np.array([data])
     else:
         raise DataValidationError(
             f"Cannot convert {type(data).__name__} to numpy array. "
-            f"Supported types: np.ndarray, Tensor, list, tuple, int, float.",
+            f"Supported types: np.ndarray, Tensor, pandas.DataFrame, polars.DataFrame, list, tuple, int, float.",
             details=f"Input type: {type(data).__name__}, value: {data}"
         )
     
@@ -257,6 +328,40 @@ def log_tensor_stats(
     )
 
 
+def create_default_standardization_arrays(
+    n_series: int,
+    dtype: Optional[np.dtype] = None
+) -> Tuple[np.ndarray, np.ndarray]:
+    """Create default standardization arrays.
+    
+    Creates arrays representing standardized data (mean=0, std=1).
+    Used internally when data is already standardized and no scaler is provided.
+    
+    Parameters
+    ----------
+    n_series : int
+        Number of series
+    dtype : np.dtype, optional
+        Data type for arrays. If None, uses DEFAULT_DTYPE.
+        
+    Returns
+    -------
+    Tuple[np.ndarray, np.ndarray]
+        (mean_array, std_array) both of shape (n_series,)
+        mean_array: zeros (standardized data has zero mean)
+        std_array: ones (standardized data has unit variance)
+    """
+    from ..config.constants import DEFAULT_DTYPE
+    
+    if dtype is None:
+        dtype = DEFAULT_DTYPE
+    
+    mean_array = np.zeros(n_series, dtype=dtype)
+    std_array = np.ones(n_series, dtype=dtype)
+    
+    return mean_array, std_array
+
+
 def select_columns_by_prefix(
     df: Any,
     prefixes: List[str],
