@@ -1,12 +1,12 @@
-"""Tests for config.types module."""
+"""Tests for config schema module."""
 
 import pytest
-from dfm_python.config.types import (
+from dfm_python.config import (
     DFMConfig,
     DDFMConfig,
     KDFMConfig,
-    SeriesConfig
 )
+from dfm_python.config.constants import DEFAULT_KDFM_AR_ORDER, DEFAULT_KDFM_MA_ORDER
 
 
 class TestDFMConfig:
@@ -14,17 +14,17 @@ class TestDFMConfig:
     
     def test_dfm_config_initialization(self):
         """Test DFMConfig can be initialized with default values."""
-        config = DFMConfig()
+        # DFMConfig requires at least one block
+        config = DFMConfig(blocks={'block1': {'num_factors': 2, 'series': []}})
         assert config is not None
     
     def test_dfm_config_parameters(self):
         """Test DFMConfig parameter setting."""
         config = DFMConfig(
-            num_factors=3,
+            blocks={'block1': {'num_factors': 2, 'series': []}},
             max_iter=100,
             threshold=1e-5
         )
-        assert config.num_factors == 3
         assert config.max_iter == 100
         assert config.threshold == 1e-5
 
@@ -34,8 +34,9 @@ class TestDDFMConfig:
     
     def test_ddfm_config_initialization(self):
         """Test DDFMConfig can be initialized."""
-        # TODO: Implement test
-        pass
+        config = DDFMConfig()
+        assert config is not None
+        assert config.clock == 'm'  # Default clock frequency
 
 
 class TestKDFMConfig:
@@ -43,15 +44,13 @@ class TestKDFMConfig:
     
     def test_kdfm_config_initialization(self):
         """Test KDFMConfig can be initialized."""
-        # TODO: Implement test
-        pass
+        config = KDFMConfig()
+        assert config is not None
+        assert config.clock == 'm'  # Default clock frequency
+        assert config.ar_order == DEFAULT_KDFM_AR_ORDER  # Default AR order from constants
+        assert config.ma_order == DEFAULT_KDFM_MA_ORDER  # Default MA order from constants
+        assert config.ar_order == 1
+        assert config.ma_order == 0
 
 
-class TestSeriesConfig:
-    """Test suite for SeriesConfig."""
-    
-    def test_series_config_initialization(self):
-        """Test SeriesConfig can be initialized."""
-        # TODO: Implement test
-        pass
 
