@@ -15,7 +15,7 @@ def sample_data():
     n_samples = 100
     n_series = 5
     
-    dates = pd.date_range(start='2020-01-01', periods=n_samples, freq='M')
+    dates = pd.date_range(start='2020-01-01', periods=n_samples, freq='ME')  # 'ME' replaces deprecated 'M'
     data = np.random.randn(n_samples, n_series)
     df = pd.DataFrame(data, index=dates, columns=[f'series_{i}' for i in range(n_series)])
     
@@ -25,13 +25,13 @@ def sample_data():
 @pytest.fixture
 def sample_config():
     """Create a basic DFMConfig for testing."""
+    # DFMConfig requires blocks with at least one block
+    # Also needs frequency dict or columns for validation
     config = DFMConfig(
-        num_factors=2,
         max_iter=10,
         threshold=1e-4,
-        nan_method=1,
-        nan_k=3,
-        mixed_freq=False
+        blocks={'block1': {'num_factors': 2, 'series': ['series_0', 'series_1']}},
+        frequency={'series_0': 'm', 'series_1': 'm'}
     )
     return config
 
