@@ -37,9 +37,10 @@ class TestBaseFactorModel:
         assert callable(kdfm.get_result)
         
         # All should have result property (preferred way to access results)
-        assert hasattr(dfm, 'result')
-        assert hasattr(ddfm, 'result')
-        assert hasattr(kdfm, 'result')
+        # Check property exists without accessing it (accessing raises if not trained)
+        assert 'result' in dir(dfm)
+        assert 'result' in dir(ddfm)
+        assert 'result' in dir(kdfm)
         # result is a property, not directly callable, but accessible via attribute access
         
         # All should have reset method (concrete in base)
@@ -322,63 +323,63 @@ class TestBaseFactorModel:
     # VAR(2) tests removed - factors now always use AR(1) dynamics (simplified)
     
     def test_validate_ndarray_with_valid_1d_array(self):
-        """Test _validate_ndarray accepts valid 1D numpy array."""
-        dfm = DFM()
+        """Test validate_ndarray_ndim accepts valid 1D numpy array."""
+        from dfm_python.numeric.validator import validate_ndarray_ndim
         arr = np.array([1.0, 2.0, 3.0])
         # Should not raise
-        dfm._validate_ndarray(arr, "test_array", 1)
+        validate_ndarray_ndim(arr, "test_array", 1)
     
     def test_validate_ndarray_with_valid_2d_array(self):
-        """Test _validate_ndarray accepts valid 2D numpy array."""
-        dfm = DFM()
+        """Test validate_ndarray_ndim accepts valid 2D numpy array."""
+        from dfm_python.numeric.validator import validate_ndarray_ndim
         arr = np.array([[1.0, 2.0], [3.0, 4.0]])
         # Should not raise
-        dfm._validate_ndarray(arr, "test_array", 2)
+        validate_ndarray_ndim(arr, "test_array", 2)
     
     def test_validate_ndarray_with_non_numpy_array(self):
-        """Test _validate_ndarray raises DataValidationError for non-numpy array."""
-        dfm = DFM()
+        """Test validate_ndarray_ndim raises DataValidationError for non-numpy array."""
+        from dfm_python.numeric.validator import validate_ndarray_ndim
         arr = [1.0, 2.0, 3.0]  # Python list, not numpy array
         
         with pytest.raises(DataValidationError, match="test_array must be 1D numpy array"):
-            dfm._validate_ndarray(arr, "test_array", 1)
+            validate_ndarray_ndim(arr, "test_array", 1)
     
     def test_validate_ndarray_with_wrong_ndim(self):
-        """Test _validate_ndarray raises DataValidationError for wrong number of dimensions."""
-        dfm = DFM()
+        """Test validate_ndarray_ndim raises DataValidationError for wrong number of dimensions."""
+        from dfm_python.numeric.validator import validate_ndarray_ndim
         arr = np.array([1.0, 2.0, 3.0])  # 1D array
         
         with pytest.raises(DataValidationError, match="test_array must be 2D numpy array"):
-            dfm._validate_ndarray(arr, "test_array", 2)
+            validate_ndarray_ndim(arr, "test_array", 2)
     
     def test_validate_ndarray_with_none(self):
-        """Test _validate_ndarray raises DataValidationError for None."""
-        dfm = DFM()
+        """Test validate_ndarray_ndim raises DataValidationError for None."""
+        from dfm_python.numeric.validator import validate_ndarray_ndim
         
         with pytest.raises(DataValidationError, match="test_array must be 1D numpy array"):
-            dfm._validate_ndarray(None, "test_array", 1)
+            validate_ndarray_ndim(None, "test_array", 1)
     
     def test_validate_ndarray_with_scalar(self):
-        """Test _validate_ndarray raises DataValidationError for scalar."""
-        dfm = DFM()
+        """Test validate_ndarray_ndim raises DataValidationError for scalar."""
+        from dfm_python.numeric.validator import validate_ndarray_ndim
         scalar = 5.0
         
         with pytest.raises(DataValidationError, match="test_array must be 1D numpy array"):
-            dfm._validate_ndarray(scalar, "test_array", 1)
+            validate_ndarray_ndim(scalar, "test_array", 1)
     
     def test_validate_ndarray_with_tuple(self):
-        """Test _validate_ndarray raises DataValidationError for tuple."""
-        dfm = DFM()
+        """Test validate_ndarray_ndim raises DataValidationError for tuple."""
+        from dfm_python.numeric.validator import validate_ndarray_ndim
         arr = (1.0, 2.0, 3.0)
         
         with pytest.raises(DataValidationError, match="test_array must be 1D numpy array"):
-            dfm._validate_ndarray(arr, "test_array", 1)
+            validate_ndarray_ndim(arr, "test_array", 1)
     
     def test_validate_ndarray_with_3d_array_when_2d_expected(self):
-        """Test _validate_ndarray raises DataValidationError for 3D array when 2D expected."""
-        dfm = DFM()
+        """Test validate_ndarray_ndim raises DataValidationError for 3D array when 2D expected."""
+        from dfm_python.numeric.validator import validate_ndarray_ndim
         arr = np.array([[[1.0, 2.0], [3.0, 4.0]], [[5.0, 6.0], [7.0, 8.0]]])  # 3D array
         
         with pytest.raises(DataValidationError, match="test_array must be 2D numpy array"):
-            dfm._validate_ndarray(arr, "test_array", 2)
+            validate_ndarray_ndim(arr, "test_array", 2)
 
