@@ -138,7 +138,7 @@ class TestKDFM:
     def test_kdfm_result_property_not_trained(self):
         """Test KDFM result property raises error when model not trained."""
         model = KDFM(ar_order=1, ma_order=0)
-        with pytest.raises(ModelNotTrainedError, match="model has not been fitted yet"):
+        with pytest.raises(ModelNotTrainedError, match="model has not been trained yet"):
             _ = model.result
     
     def test_kdfm_predict_requires_trained_model(self):
