@@ -19,12 +19,13 @@ import numpy as np
 from datetime import datetime
 from dfm_python import DDFM, DDFMDataModule, DDFMTrainer
 from dfm_python.config import DDFMConfig
+from dfm_python.config.constants import TUTORIAL_MAX_PERIODS, DEFAULT_LEARNING_RATE, DEFAULT_BATCH_SIZE, DEFAULT_DDFM_LEARNING_RATE
 from dfm_python.utils.misc import TimeIndex
-
-# sktime imports for preprocessing
-from sktime.transformations.compose import TransformerPipeline
-from sktime.transformations.series.impute import Imputer
+from sklearn.pipeline import Pipeline
 from sklearn.preprocessing import StandardScaler
+from sktime.transformations.series.impute import Imputer
+
+# Preprocessing pipeline helper (uses sktime internally)
 
 print("=" * 80)
 print("DDFM Tutorial: Macro Data")
@@ -85,12 +86,11 @@ df_processed = df_with_date.drop(columns=["date"]).copy()
 df_processed = df_processed.dropna(how='all')
 
 # Use only recent data for faster execution
-# Take last 100 periods (further reduced for faster execution)
-max_periods = 100
-if len(df_processed) > max_periods:
-    df_processed = df_processed.iloc[-max_periods:]
-    date_column = date_column.iloc[-max_periods:]
-    print(f"   Using last {max_periods} periods for faster execution")
+# Take last TUTORIAL_MAX_PERIODS periods (reduced for faster execution)
+if len(df_processed) > TUTORIAL_MAX_PERIODS:
+    df_processed = df_processed.iloc[-TUTORIAL_MAX_PERIODS:]
+    date_column = date_column.iloc[-TUTORIAL_MAX_PERIODS:]
+    print(f"   Using last {TUTORIAL_MAX_PERIODS} periods for faster execution")
 
 print(f"   Data shape after cleaning: {df_processed.shape}")
 
@@ -103,49 +103,53 @@ print(f"   Missing values before preprocessing: {missing_before} ({missing_befor
 # ============================================================================
 print("\n[Step 2.5] Creating preprocessing pipeline with sktime...")
 
-# Simplified preprocessing: Apply difference to target series manually, then use unified pipeline
-# This is faster than ColumnEnsembleTransformer for small datasets
-
-# Apply difference transformation to target series manually (for chg transformation)
-if target_col in df_processed.columns:
-    target_idx = df_processed.columns.get_loc(target_col)
-    target_series = df_processed[target_col].values
-    # Apply first difference
-    target_diff = np.diff(target_series, prepend=target_series[0])
-    df_processed[target_col] = target_diff
-    print(f"   Applied difference transformation to {target_col}")
+# Note: Target series will be kept raw (no differencing, no preprocessing pipeline)
 
 # Date column is already separated, so df_processed has no date column
 # Use df_processed directly for preprocessing
 df_for_preprocessing = df_processed
 
-# Create simplified preprocessing pipeline: Imputation → Scaling
-# (Transformations already applied manually above)
-# This pipeline will be fitted and used to preprocess feature data
-# Target series will be handled separately (not preprocessed by this pipeline)
-preprocessing_pipeline = TransformerPipeline(
+# Separate X (features) and y (target)
+X_cols = [col for col in selected_cols if col != target_col]
+y_col = target_col
+
+X = df_for_preprocessing[X_cols].copy()
+y = df_for_preprocessing[[y_col]].copy()  # Keep y raw (no preprocessing)
+
+print("   Separating features (X) and target (y)...")
+print(f"   X (features): {len(X_cols)} series - will be preprocessed")
+print(f"   y (target): 1 series ({y_col}) - kept raw (no preprocessing pipeline, no differencing)")
+
+# Create preprocessing pipeline for X (features): Imputation → Scaling
+X_pipeline = Pipeline(
     steps=[
-        ('impute_ffill', Imputer(method="ffill")),  # Forward fill missing values
-        ('impute_bfill', Imputer(method="bfill")),  # Backward fill remaining NaNs
-        ('scaler', StandardScaler())  # Unified scaling for all series
+        ('impute_ffill', Imputer(method="ffill")),
+        ('impute_bfill', Imputer(method="bfill")),
+        ('scaler', StandardScaler())
     ]
 )
 
-def _get_fitted_scaler(pipeline, data_frame):
-    """Extract fitted scaler; if not fitted, fit it on provided data."""
-    steps_attr = getattr(pipeline, "steps_", None)
-    candidate = (steps_attr or pipeline.steps)[-1][1]
-    if not hasattr(candidate, "n_features_in_"):
-        candidate = candidate.fit(data_frame)
-    return candidate
+print("   Pipeline for X: Imputer(ffill) → Imputer(bfill) → StandardScaler")
+print("   y (target): Raw series (no preprocessing pipeline)")
+print("   Applying preprocessing pipeline to X only...")
 
-print("   Pipeline: Imputer(ffill) → Imputer(bfill) → StandardScaler")
-print(f"   Transformations: {target_col} uses difference (chg), others use linear")
-print("   Applying preprocessing pipeline...")
+# Fit and transform X (features)
+X_pipeline.fit(X)
+X_preprocessed = X_pipeline.transform(X)
 
-# Fit preprocessing pipeline on training data once (without date column)
-fitted_pipeline = preprocessing_pipeline.clone().fit(df_for_preprocessing)
-df_preprocessed = fitted_pipeline.transform(df_for_preprocessing)
+# Create scaler for y (target) - fit but don't transform
+# This scaler will be used for inverse transformation during prediction
+y_scaler = StandardScaler()
+y_scaler.fit(y)
+
+# Ensure X output is DataFrame
+if isinstance(X_preprocessed, np.ndarray):
+    X_preprocessed = pd.DataFrame(X_preprocessed, columns=X_cols, index=X.index)
+else:
+    X_preprocessed = pd.DataFrame(X_preprocessed, columns=X_cols, index=X.index)
+
+# Combine X (preprocessed) and y (raw) back together
+df_preprocessed = pd.concat([X_preprocessed, y], axis=1)
 
 # Ensure output is DataFrame
 if isinstance(df_preprocessed, np.ndarray):
@@ -197,8 +201,9 @@ config = DDFMConfig(
     factor_order=1,  # VAR(1) - first-order autoregressive
     encoder_layers=[32, 16],  # Reduced for faster execution
     epochs=10,  # Reduced for faster execution
-    learning_rate=0.001,
-    batch_size=32
+    learning_rate=DEFAULT_LEARNING_RATE,
+    batch_size=DEFAULT_BATCH_SIZE,
+    target_scaler=y_scaler  # Fitted scaler for target series inverse transformation
 )
 
 print(f"   Number of series: {len(selected_cols)}")
@@ -242,8 +247,8 @@ model = DDFM(
     factor_order=1,  # VAR(1) - first-order factor dynamics
     epochs=10,  # Reduced for faster execution
     max_iter=3,  # Reduced for faster execution
-    batch_size=32,  # Reduced for faster execution
-    learning_rate=0.005
+    batch_size=DEFAULT_BATCH_SIZE,  # Reduced for faster execution
+    learning_rate=DEFAULT_DDFM_LEARNING_RATE
 )
 model._config = config  # Set config directly
 
@@ -257,41 +262,12 @@ print("   Training completed!")
 # ============================================================================
 print("\n[Step 6] Making predictions...")
 
-X_forecast = None
-Z_forecast = None
-# Exclude date column for scaler extraction
-df_for_scaler = df_processed.drop(columns=['date']) if 'date' in df_processed.columns else df_processed
-scaler = _get_fitted_scaler(fitted_pipeline, df_for_scaler)
+# Predict with horizon=6 (uses target_series from DataModule)
+X_forecast, Z_forecast = model.predict(horizon=6)
 
-try:
-    # Predict with default horizon (uses target_series from DataModule)
-    X_forecast, Z_forecast = model.predict(horizon=6)
-    
-    print(f"   Forecast shape: {X_forecast.shape} (target series only)")
-    print(f"   Factor forecast shape: {Z_forecast.shape}")
-    
-    # X_forecast now contains only target series (no features)
-    if X_forecast.shape[1] == 1:
-        print(f"   First forecast value (target {target_col}): {X_forecast[0, 0]:.6f}")
-    else:
-        print(f"   First forecast values (targets): {X_forecast[0, :]}")
-    
-    # Note: Inverse-transform check removed - predict() now returns only target series,
-    # so full-series scaler cannot be used for validation
-    
-    # Predict with history parameter (using recent 60 periods)
-    # History-based prediction removed - use standard predict()
-    X_forecast, Z_forecast = model.predict(horizon=6)
-    
-    # History-based prediction removed - use standard predict()
-    # Note: Inverse-transform check removed - predict() now returns only target series
-    
-except ValueError as e:
-    print(f"   Prediction failed: {e}")
-    print("   Note: This may indicate numerical instability. Try:")
-    print("   - Using more training iterations")
-    print("   - Adjusting data transformations")
-    print("   - Using different factor configurations")
+print(f"   Forecast shape: {X_forecast.shape}")
+print(f"   Factor forecast shape: {Z_forecast.shape}")
+print(f"   First forecast value (target): {X_forecast[0, 0]:.6f}")
 
 # ============================================================================
 # Step 7: Summary
@@ -300,7 +276,7 @@ print("\n" + "=" * 80)
 print("Tutorial Summary")
 print("=" * 80)
 print(f"✅ Data loaded: {df.shape[0]} rows, {len(selected_cols)} series")
-print(f"✅ Model trained: {len(series_configs)} series, 1 factor, VAR(1) dynamics")
+print(f"✅ Model trained: {len(selected_cols)} series, 1 factor, VAR(1) dynamics")
 if X_forecast is not None:
     print(f"✅ Predictions generated: {X_forecast.shape[0]} periods ahead")
 else:
